
#include <windows.h>
#include <stdio.h>
#include "game.h"

#define VK_N		'N'	
#define VK_P		'P'
#define VK_Q		'Q'
//#define VK_ENTER	'\n'
#define VK_Y		'Y'

bool keys[256];								// keystates
HFONT font;									// Font struct
BUFFER gBuffer;								// Create the double buffering structure
HBITMAP gBitmaps[30];						// Create bitmap array, current size is 20
Entity playership;							// Playership object
Entity pBullets[MAX_BULLETS];				// Player bullet array
Entity pExplosions;
Entity asteroids[MAX_ASTEROIDS];			// Asteroids array
Entity enemyship[MAX_ENEMYSHIP];			// Enemy ship array
Entity enemyship2[MAX_ENEMYSHIP2];			// Enemy ship type 2 array
Entity eBullets[MAX_ENEMYBULLETS];			//Enemy ship ammo
Entity explosion[MAX_NUMBER_EXPLOSIONS];	//How many explosions that can happen simultanouesly
Entity bonusScore[MAX_BONUS_SCORE];		    // Health bonuses
Entity hazard[MAX_HAZARD];					// Hazard bonus
Entity life[3];								// Life entity, Max 3

// The healthbar objects
Entity fullhealth;
Entity health80;
Entity health60;
Entity health40;
Entity health20;
Entity healthzero;
Entity introback;							// Intro bitmap
Entity text;								// also used in Intro
Entity pause;								// pause screen bitmap


int game_state = GAME_INTRO;
int bullet_countdown= 3;
int bullet_countdown_enemy= 0;

int Spawntimer=30; // Delay for spawning stuff after player resets
int timer2 = 30; //Do not change for the moment, used for final playership explosion delay time 
int gYScroll = 0;
int gScore = 0; //Player score
int gHealth = 300;
int countdown = 0;
bool explodedOnce = false;
bool explodedTwice = false;
bool explodedThrice = false;

int BigExplodeDelay = 25; // Player ship explosion delay before starting new loop
char texttype[100]; // for our printStuff function how many strings to store
char textout[500];


// --------------------------------------------------------------------
// Entity Struct Functions
// --------------------------------------------------------------------

void entityInit(Entity &ent, int x, int y, int vx, int vy, bool alive, int health, int tileWidth, int tileHeight,float aniRate, HBITMAP bitmap ) // Initialise the Entity struct
{
	ent.x= x;
	ent.y= y;
	ent.vx = vx;
	ent.vy = vy;
	ent.health = health;
	ent.alive = alive;
	ent.bitmap = bitmap;		
	ent.aniRate = aniRate;
	ent.tileHeight= tileHeight;
	ent.tileWidth = tileWidth;
}

void entityAnimate(Entity &ent)
{
	int tnx; int tny;
	SIZE size = GetBitmapSize(ent.bitmap);	
	tnx = size.cx/ent.tileWidth;
	tny = size.cy/ent.tileHeight;	
	ent.aniIndex += ent.aniRate;
	if(ent.aniIndex >= tnx*tny) //! Check if it reached the last frame
	{
		ent.aniIndex = 0;      //!Reset animation to start 
	}
}

void entityAnimateOnce(Entity &ent)
{
	int tnx; int tny;
	SIZE size = GetBitmapSize(ent.bitmap);	
	tnx = size.cx/ent.tileWidth;
	tny = size.cy/ent.tileHeight;	
	ent.aniIndex += ent.aniRate;
	if(ent.aniIndex >= tnx*tny) //! Check if it reached the last frame
	{
		ent.aniIndex = 0;      //!Reset animation to start 
		//ent.aniRate = 0; //!Stops the counter **needs some fixing, if you use multiple instance of this function it bugs out
		ent.alive = 0; //!Removes the object
	}
}

void entityMove(Entity &ent)
{
	ent.x += ent.vx;
	ent.y += ent.vy;
}

void entityRender(const Entity &ent)
{
	if(ent.alive)
	{
		int tnx; int tny;
		SIZE size = GetBitmapSize(ent.bitmap);
		tnx = size.cx/ent.tileWidth;
		tny = size.cy/ent.tileHeight;	
		int id = (int)ent.aniIndex;
		RECT rc = {(id%tnx)*ent.tileWidth, (id/tnx)*ent.tileHeight, (id%tnx+1)*ent.tileWidth, (id/tnx+1)*ent.tileHeight}; 
		DisplayTransparentBitmap(&gBuffer, ent.bitmap, ent.x, ent.y, rc);
	}
	
}

// -------------------------- End of function for the Entity struct-------------------------------


// --------------------------------------------------------------------
// General System and Game function
// --------------------------------------------------------------------

//! Initialize system variables
void SystemInit(HWND hwnd)
{	
	// Create our double buffering
	CreateDoubleBuffering(&gBuffer, hwnd);			

	// Clear the backbuffer to white
	ClearScreen(gBuffer.hdcBack, gBuffer.scrnRect, WHITE_BRUSH);
}

//! Clean up system variables
void SystemDeInit()
{
	// Set the back buffer to it's original bitmap (This is necessary for clean-up)
	SelectObject(gBuffer.hdcBack, gBuffer.hOldBitmap);

	// Post the QUIT message to the window (0 = WM_QUIT)															
	PostQuitMessage(0);				
}	

// Checks the frame rate in "frame_rate"
// Returns true if the frame rate is ok
bool LockFrameRate(int frame_rate)
{
	static float lastTime = 0.0f;
	
	// Get current time in seconds (milliseconds * .001 = seconds)
	float currentTime = GetTickCount() * 0.001f; 

	// Get the elapsed time by subtracting the current time from the last time
	// If the desired frame rate amount of seconds has passed -- return true (ie Blit())
	if((currentTime - lastTime) > (1.0f / frame_rate))
	{
		// Reset the last time
		lastTime = currentTime;
			return true;
	}

	return false;
}

//! Returns the width of \a rect;
long GetRectWidth(const RECT &rect)
{
	return rect.right - rect.left;
}

//! Returns the height of \a rect
long GetRectHeight(const RECT &rect)
{
	return rect.bottom - rect.top;
}

// Struct to get the size of the bitmap with the member size
SIZE GetBitmapSize(HBITMAP hbm)
{
	BITMAP bm;
	GetObject(hbm, sizeof(bm), &bm);
	SIZE size= { bm.bmWidth, bm.bmHeight };
	return size;
}
//-----------------------------End of General System and Game functions-------------------------

// --------------------------------------------------------------------
// Game specific functions
// --------------------------------------------------------------------


// --------------------------------------------------------------------
// Game States
// --------------------------------------------------------------------
//! Actions for a single game frame
void GameFrame()
{
	switch(game_state)
	{
	case GAME_INTRO:
		{
		intro();
		if (keys[VK_SPACE]) game_state = GAME_RUN;
		if (keys[VK_Q]) game_state = GAME_EXIT;			
		}break;

	case GAME_INIT: 
		{
			PlayerReset();
			ResetAll();
			gScore = 0; // Resets score
			gHealth = 300; // Resets health		
			game_state = GAME_RUN;
			GameInit();
		}break;

	case GAME_RUN: // Make everything live
		{			
			if (keys[VK_P]) game_state = GAME_STATE_GAME_PAUSE;
			if (keys[VK_Q]) game_state = GAME_EXIT;
			ScrollBackground(gBuffer); // Scrolling background					
			InitLife(); // Inits the three cute hearts that represent lives 
			
			if (playership.alive == true){ // if Player is alive	
							
				printStuff(10, 510, "Score : %d", RGB(255,0,255), 20, "Arial Bold", 700, TRUE, 0, gScore); // Prints the score
				healthbar(); // Our healthbar 
				printStuff(250, 510, "Health", RGB(255,0,255), 14, "Arial Bold", 400, 0, 0, 0); // prints the word health next to the health bar
				BigExplodeDelay = 25; // resets the player explosion counter
				PlayerSpawn();

				SetEnemy1Bullet();	
				SetEnemy2Bullet();

				LifeCounter();
				Spawntimer--;	// This gives a slight delay before spawning the enemies and asteroids each reset so player won't get swamped
				if(Spawntimer == 0){				
				EnemySpawn();
				Enemy2Spawn();
				AsteroidSpawn();				
				Spawntimer=30;
				}
				SpawnBonus();
				SpawnExplosion();				

				MoveBullets(); // Player bullets
				MoveEnemyBullets();

				boundaryCheck(playership); // Make sure player ship don't go out of screen	
				RoidPlayerCol(); // Asteroids and player collision detection and response
				PlayerBulAsteroidCol(); // Checks for playerbullet and asteroid collision, blows up an asteroid if so	
				PlayerBulEnemy1Col(); // Checks to see if player shot and enemy
				PlayerBulEnemy2Col();				
				Enemy1BulletPlayerCol();
				Enemy2BulletPlayerCol();
				EnemyPlayerCol();
				Enemy2PlayerCol();
				PlayerExplosionCount();

				PlayerBonus1(); // Check if player gets the health bonus and act accordingly
				PlayerBonus2(); // Check if player gets the hazard bonus and act accordingly

				GameRender();
			}
			if(playership.alive == false){ // this sets the small delay after each player explosion
				int m;
				for (m=0; m<MAX_NUMBER_EXPLOSIONS; m++){
					explosion[m].alive = false;
				}
				BigExplodeDelay--;				
				entsAnimateOnce(&pExplosions, 1);
				
				int i,j,k,l;
					for (i=0; i<MAX_ASTEROIDS; i++){						
						asteroids[i].alive = false;
					}
					for (j=0; j<MAX_ENEMYSHIP; j++){						
						enemyship[j].alive = false;
					}
					for (k=0; k<MAX_ENEMYSHIP; k++){
						enemyship2[j].alive = false;
					}
					for (l=0; l<MAX_ENEMYBULLETS; l++){
						eBullets[l].alive = false;
					}						
					if (BigExplodeDelay == 0){			
						playership.alive = true;
						pExplosions.alive = false;
					}
			}
		}break;

	case GAME_STATE_GAME_PAUSE:
		{
			ScrollBackground(gBuffer);			
			entityRender(pause);		
			PauseAll();
			// if <enter> key pressed resume game
			if (keys[VK_RETURN]) game_state = GAME_RUN;
		} break;

	case GAME_OVER:
		{		
			if (keys[VK_Y]) game_state = GAME_INTRO;
			if (keys[VK_N]) game_state = GAME_EXIT;   
			ShowScore();
		}break;		

	case GAME_EXIT:
		{
		GameDeInit();
		SystemDeInit();
		PostQuitMessage(0);
		}break;
	}
	
}
//-----------------------------End of Game States---------------------------

// --------------------------------------------------------------------
// Game Initilisation / De-initialisation
// --------------------------------------------------------------------

//! Initialize game objects and bitmaps
bool GameInit()
{
	memset(gBitmaps, NULL, 30*sizeof(HBITMAP)); // to empty our array
	gBitmaps[0] = LoadABitmap("toss_arbiter.bmp"); // load in the bitmaps into gBitmaps array	
	gBitmaps[1] = LoadABitmap("background.bmp");
	gBitmaps[2] = LoadABitmap("bullet1.bmp");
	gBitmaps[3] = LoadABitmap ("asteroid.bmp");
	gBitmaps[4] = LoadABitmap ("terran_dropship.bmp");
	gBitmaps[5] = LoadABitmap ("terran_wraith.bmp");
	gBitmaps[6] = LoadABitmap ("bullet2.bmp");
	gBitmaps[7] = LoadABitmap ("explosion2.bmp");
	gBitmaps[8] = LoadABitmap ("bonusscore.bmp");
	gBitmaps[9] = LoadABitmap ("hazard.bmp");
	gBitmaps[10] = LoadABitmap ("life.bmp");
	gBitmaps[11] = LoadABitmap ("explode1.bmp");
	gBitmaps[12] = LoadABitmap ("fullhealth.bmp");
	gBitmaps[13] = LoadABitmap ("health80.bmp");
	gBitmaps[14] = LoadABitmap ("health60.bmp");
	gBitmaps[15] = LoadABitmap ("health40.bmp");
	gBitmaps[16] = LoadABitmap ("health20.bmp");
	gBitmaps[17] = LoadABitmap ("health0.bmp");
	gBitmaps[18] = LoadABitmap ("introscreen.bmp");
	gBitmaps[19] = LoadABitmap ("pause.bmp");

	int i;
	for(i=0;i<20;i++){
		if(!gBitmaps[i]){	
			char number[20];
			sprintf(number, "%d",i);
			MessageBox(NULL, "Can't find the tile bitmap!", "Error", MB_OK);
			PostQuitMessage(0);
		}
	}

	//if(!gBitmaps[0]||!gBitmaps[1]||!gBitmaps[2]||!gBitmaps[3]||!gBitmaps[4]||!gBitmaps[5]||!gBitmaps[6]||!gBitmaps[7]||!gBitmaps[8]||!gBitmaps[9]||
	//	!gBitmaps[10]||!gBitmaps[11]||!gBitmaps[12]||!gBitmaps[13]||!gBitmaps[14]||!gBitmaps[15]||!gBitmaps[16]||!gBitmaps[17]
	//	||!gBitmaps[18]||!gBitmaps[19]) {
	//	MessageBox(NULL, "Can't find the tile bitmap!", "Error", MB_OK);
	//	PostQuitMessage(0);
	//}// debug line in case bitmap doesn't exist

	//! Initialisation for our game objects
		
	//! Fills the bullet array with initial value
	for (i=0;i<MAX_BULLETS;i++){
		entityInit(pBullets[i], playership.x+15, playership.y, 0, 0,false, 100,20,20,0 ,gBitmaps[2]);
	} 
	for (i=0;i<MAX_ENEMYBULLETS;i++){
		entityInit(eBullets[i], 0, 0, 0, 0, false, 100,20,20,0 ,gBitmaps[6]);
	}		
	//! Fills the asteroid array with inital value
	for (i=0;i<MAX_ASTEROIDS;i++){
		entityInit(asteroids[i], WIN_WID - rand()%(WIN_WID+30), WIN_HGT-rand()%(WIN_HGT+40),rand()%16,rand()%16,false,100, 64,64,0.8, gBitmaps[3]);
	}	
	//! Enemy ship array (there are two types of ship at the moment)
	for (i=0;i<MAX_ENEMYSHIP;i++){
		entityInit(enemyship[i], 0,0,6,6,false,100,44,43,0,gBitmaps[4]);
	}
	for (i=0;i<MAX_ENEMYSHIP2;i++){
		entityInit(enemyship2[i], 300,300,5,5,false,100,47,40,0,gBitmaps[5]);
	}
	for (i=0;i<MAX_NUMBER_EXPLOSIONS;i++){
	entityInit(explosion[i], 0,0,0,0,false, 100,64,64, 0.6, gBitmaps[7]); //!Explosion sequence
	}	
	for (i=0;i<MAX_BONUS_SCORE;i++){
	entityInit(bonusScore[i], 200,200,0,0,false, 100,80, 40, 0.5, gBitmaps[8]); //!Explosion sequence
	}
	for (i=0;i<MAX_HAZARD;i++){
	entityInit(hazard[i], 200,200,0,0,false, 100,40,36, 0.8, gBitmaps[9]); //!Explosion sequence
	}
	for (i=0;i<3;i++){
	entityInit(life[i], 460+(i*50),580,0,0, true, 100,40,39, 0, gBitmaps[10]); //! Player life
	}	
	entityInit(playership, 350, 300, 0, 0, true, 300,48,49,0, gBitmaps[0]); //! init player ship
	entityInit(pExplosions, 200,200,0,0,false, 100,64,64, 0.7, gBitmaps[11]); //! Player explosion

	entityInit(fullhealth, 320,500,0,0,false, 100,203,44, 0, gBitmaps[12]); //! Player explosion
	entityInit(health80, 320,500,0,0,false, 100,203,44, 0, gBitmaps[13]);
	entityInit(health60, 320,500,0,0,false, 100,203,44, 0, gBitmaps[14]);
	entityInit(health40, 320,500,0,0,false, 100,203,44, 0, gBitmaps[15]);
	entityInit(health20, 320,500,0,0,false, 100,203,44, 0, gBitmaps[16]);
	entityInit(healthzero, 320,500,0,0,false, 100,203,44, 0, gBitmaps[17]);		
	entityInit(introback, 100, 25, 0, 0, true, 100, 600, 480,0, gBitmaps[18]); //! The intro header		
	entityInit(pause, 280, 200, 0, 0, true, 100, 269, 131, 0, gBitmaps[19]); //! The intro header
	return true;
}

void intro()
{
	ScrollBackground(gBuffer);	
	entityRender(introback);
	entityRender(text);		
}

void InitLife() // Initialises the life counter (set to 3)
{
	life[0].x = 640;
	life[0].y = REAL_BOTTOM;
	life[0].alive = true;
	life[1].x = 690;
	life[1].y = REAL_BOTTOM;
	life[1].alive = true;
	life[2].x = 740;
	life[2].y = REAL_BOTTOM;
	life[2].alive = true;
}

//! Cleans up game objects and bitmaps
void GameDeInit()
{
	int i;
	for (i=0;i<19;i++){
	DeleteObject(gBitmaps[i]);	
	}
}

//-------------------------------------------------End of Game Init/De-init------------------------------------------------


// --------------------------------------------------------------------
// Game AI
// --------------------------------------------------------------------

void LifeCounter() // deducts one life everytime player health reaches zero				   
{
	if(gHealth <= 200 && gHealth >100){
		life[2].alive = false;
	}
	if(gHealth <= 100 && gHealth >1){
		life[1].alive = false;
		life[2].alive = false;
	}
	if(gHealth <= 0 && gHealth <100){
		life[0].alive = false;
		life[1].alive = false;
		life[2].alive = false;
	}
	if(gHealth < 0){
		gHealth = 0;
	}
}

void PlayerExplosionCount() // How many times the player is blown up, also starts the delay
{
	if (gHealth <= 200 && explodedOnce == false && playership.alive == true){
		pExplosions.alive = true;
		pExplosions.x = playership.x;
		pExplosions.y = playership.y;
		explodedOnce = true;
		explodedTwice = true;
		playership.alive = false;
	}
	
	else if (gHealth <= 100 && explodedTwice == true){		
		pExplosions.alive = true;
		pExplosions.x = playership.x;
		pExplosions.y = playership.y;		
		explodedTwice = false;
		explodedThrice = true;
		playership.alive = false;
	}

	else if (gHealth == 0 && explodedThrice == true){
		
		pExplosions.alive = true;
		pExplosions.x = playership.x;
		pExplosions.y = playership.y;
		entsAnimateOnce(&pExplosions, 1);
		
		
		timer2--;
		if (timer2 == 0){
			explodedTwice = false;
			explodedThrice = false;
			explodedOnce = true;
						
			timer2=30;
			game_state = GAME_OVER;
		}	
	}

}
		


void InsertExplosion(Entity &ent) //! Places the explosion at the correct coordinates
{   
	ent.alive= false;
	int i;  	
	for (i=0; i < MAX_NUMBER_EXPLOSIONS; i++){
		if(!explosion[i].alive){
			if (explosion[i].alive == false) { //! Insert new explosion if current one is dead
				explosion[i].x= ent.x;
				explosion[i].y= ent.y;
				explosion[i].alive = true;  
				explosion[i].duration = 35;           
				return;       
			}
		}
	}
}

void SpawnExplosion() // This is for to make sure the explosion goes off after a set of time
{
	int i;	
	for (i=0; i < MAX_NUMBER_EXPLOSIONS; i++){		
			if (explosion[i].alive == true){				               
				explosion[i].duration--;
				if (explosion[i].duration == 0) {
					explosion[i].alive = false;
				}
			}
	}
}

void PauseAll() // Pause all entity thats alive except bullets (will work on it)
{
	int c, d, e, f;
			
			for (c=0; c<MAX_ENEMYSHIP; c++){
				if(enemyship[c].alive =true){
				enemyship[c].x = enemyship[c].x;
				enemyship[c].y = enemyship[c].y;
				}
			}
			for (d=0; d<MAX_ENEMYSHIP2; d++){
				if(enemyship2[d].alive = true){
				enemyship2[d].x = enemyship2[d].x;
				enemyship2[d].y = enemyship2[d].y;
				}
			}
			for (e=0; e<MAX_ASTEROIDS; e++){
				if(asteroids[e].alive = true){
				asteroids[e].x = asteroids[e].x;
				asteroids[e].y = asteroids[e].y;
				}
			}
			for (f=0;f<MAX_NUMBER_EXPLOSIONS;f++){
				explosion[f].alive = false;
				explosion[f].alive = false;
			}
}

void ScrollBackground(BUFFER &buffer) //! Background Scrolling function
{ 
	int x_pos = 0;
	int y_pos = gYScroll;	
	HDC temp_dc = CreateCompatibleDC(buffer.hdcBack);

		//! Error Check
		if(temp_dc == NULL)
			return;

	//! Load background image into emp_dc
	HBITMAP save_bmp = (HBITMAP)SelectObject(temp_dc,gBitmaps[1]);
	//! Draw the background tile going left to right, from top to bottom
	//! Once y_pos (for the upper left corner) is greater than the height of the window
	//! drawing for current frame is done
	while(y_pos < GetRectHeight(buffer.scrnRect))
	{
		
		//! Draw going left to right  			
		while(x_pos < GetRectWidth(buffer.scrnRect))
		{			
			//! Blit the background tile at the current location
			BitBlt(buffer.hdcBack,x_pos,y_pos,BKGRD_WID,BKGRD_HGT,
				   temp_dc,0,0,SRCCOPY);

			x_pos += BKGRD_WID; //! Move the X-pos of the upper left corner (of where to blit to)
								// to the right

		}

		x_pos = 0; //! Zero out x_pos 

		//! Move down to the next "row" for blitting the background tile
		y_pos += BKGRD_HGT;		
		}

	
		x_pos = 0; //! Zero out x_pos

		while(x_pos < GetRectWidth(buffer.scrnRect))
		{
			//! Blit the background tile at the current location
			BitBlt(buffer.hdcBack,x_pos,0,BKGRD_WID,gYScroll,
				   temp_dc,0,(BKGRD_HGT - gYScroll),SRCCOPY);			

			x_pos += BKGRD_WID; //! Move the X-pos of the upper left corner (of where to blit)
								// to the right

		}
	
	if(++gYScroll == BKGRD_HGT)
		gYScroll = 0; //! Set gYScroll to zero

	//! Free up the memory
	SelectObject(temp_dc,save_bmp);
	DeleteDC(temp_dc);
} 


bool boundaryCheckDel(Entity &ent){ //! Check if the object is out of our window, delete object if true
	
	if (ent.y < 0 || ent.y > (REAL_BOTTOM - ent.tileHeight)){ 
		return TRUE;
	}
	if (ent.x < (0 - ent.tileWidth/2) || ent.x > (WIN_WID -ent.tileWidth) ) { // Fix this later , dimensions need checking
		return TRUE;
	}
	return FALSE;
}

bool InsertBonusProb() // Chance to spawn bonus
{
	if (rand()%1000 < CHANCE_SPAWN_BONUS){
		return true;
	}
	else return false;
}

void InsertBonusSpawn(Entity &ent) // Randomly spawns a bonus when the player destroys enemy ship
{
	int l;int m;	
	int chance1 = (rand()%1000);
	for (l=0;l<MAX_BONUS_SCORE;l++){
		for(m=0;m<MAX_HAZARD;m++){
			if (bonusScore[l].alive == false || hazard[m].alive == false){
			if (chance1 >=500){ // Don't spawn anything	
				return;
				}
			else if (chance1 >= 250 && chance1 < 500){ // Spawn health bonus				
				bonusScore[l].alive=true;
				bonusScore[l].duration=HEALTH_BONUS_DURATION; // Set health bonus duration	
				bonusScore[l].x=ent.x;
				bonusScore[l].y=ent.y;				
				return;
			}
			else if (chance1 >= 0 && chance1 < 250){ // Spawn hazard bonus				
				hazard[m].alive=true;
				hazard[m].duration=HAZARD_BONUS_DURATION; 
				hazard[m].x=ent.x;
				hazard[m].y=ent.y;				
				return;
			}
			}
		}
	}
	
}
		
void SpawnBonus() // Spawns the bonus goodies
{
	int i;int j;
	for (i=0; i<MAX_BONUS_SCORE; i++){
		if(bonusScore[i].alive == true ){		
		bonusScore[i].duration--;
		if (bonusScore[i].duration == 0 ){
				bonusScore[i].alive = false;
				bonusScore[i].duration =0;
			}	
		}
	}
	for (j=0;j<MAX_HAZARD;j++){
		if(hazard[j].alive == true){			
			hazard[j].duration--;				
			if (hazard[j].duration == 0){
				hazard[j].alive = false;
			}
		}
	}
}

void PlayerSpawn() // Spawns the player , makes him alive again if he was killed last round
{
	if (playership.alive == false){
		playership.alive = true;
		playership.x = 400;
		playership.y= 300;
	}
	playership.vy=0; // Just in case
	playership.vx=0;
	
	if (keys[VK_UP]){playership.vy=-12;};
	if (keys[VK_DOWN]){playership.vy=12;};
	if (keys[VK_LEFT]){playership.vx=-12;};
	if (keys[VK_RIGHT]){playership.vx=12;};
	if (keys[VK_SPACE]){
		InsertBullet();		// Player fire function
	}	
}

void PlayerReset() // Resets the player paremeters
{
	playership.vy=0; // Just in case
	playership.vx=0;
	playership.x=400;
	playership.y=300;
	playership.alive = true;
	playership.health = 300;
}



void AsteroidSpawn() //! Handles the spawning of the asteroids
{
	int i;
	for (i=0;i<MAX_ASTEROIDS;i++){		// This bit adds the asteroids to the game				
		if (boundaryCheckDel(asteroids[i]) == TRUE){
			asteroids[i].alive = false;
		}	
	}	

	RoidSpawnProb();            // Calculates the probability of spawning another asteroid (so our game won't look empty)
	if (RoidSpawnProb() == true) {
		for (i=0;i<MAX_ASTEROIDS;i++){
			if (!asteroids[i].alive) {
				RoidReinit(asteroids[i]);
			}
		}
	}
}

int CountAsteroids() // counts how many active asteroids we have
 {
	 int active=0;
	 int i;
	 for (i=0;i<MAX_ASTEROIDS;i++){
		 if (asteroids[i].alive == true){
			 active++;
		 }
	 }
	 return active;
 }

bool RoidSpawnProb() // some function to see if we need to spawn more asteroid or not
{
	if( CountAsteroids() < (MAX_ASTEROIDS-rand()%MAX_ASTEROIDS)){
		return TRUE;
	}
	else return FALSE;
}

void RoidReinit(Entity &ent) // Reinitialises the asteroids after they are destroyed
{							// Change attributes like speed of roids here
	ent.alive = true;
	int chance = (rand()%1000+rand()%1000)/2;
	if (chance <= 400){ // Spawns from top at any point

		ent.x = rand()%(800-ent.tileWidth);
		ent.y = -ent.tileHeight;		
		int rand1 = rand()%10;
		if (rand1 <=5){
			ent.vx = (rand()%8)+1; 
			ent.vy = (rand()%12)+1;
		}
		if (rand1 >5){
			ent.vx = (rand()%8)+1;
			ent.vy = (rand()%12)+1;
		}
	}
	if (chance > 400 && chance <= 700){ // Spawns from left side at any point			
		ent.x =- ent.tileWidth;
		ent.y = rand()%WIN_HGT;		
		int rand1 = rand()%10;
		if (rand1 <=5){
			ent.vx = (rand()%10)+1; 
			ent.vy = -(rand()%5)+1;
		}
		if (rand1 >5){
			ent.vx = ((rand()%10)+1)*(-1);
			ent.vy = (rand()%11)+1;
		}
	}
	if (chance > 700 && chance <= 1000){ // Spawns from right side at any point
		ent.x = WIN_WID-ent.tileWidth;
		ent.y = rand()%WIN_HGT;		
		int rand1 = rand()%10;
		if (rand1 <=5){
			ent.vx = ((rand()%10)+1)*(-1); 
			ent.vy = (rand()%12)+1;
		}
		if (rand1 >5){
			ent.vx = ((rand()%10)+1)*(-1);
			ent.vy = (rand()%11)+1;
		}
	}
	ent.health = 100;
}

void EnemySpawn() //! Handles the spawning of enemyship
{
	int i, j;
	
	for (i=0; i<MAX_ENEMYSHIP; i++){					
		    if (boundaryCheckDel(enemyship[i]) == TRUE){
				enemyship[i].alive = false;
			}		
	}	
	EnemySpawnProb();  	
	if (EnemySpawnProb() == true) {			
		int howmanytospawn = (rand()%MAX_ENEMYSHIP)-1;
			for (i=0;i<howmanytospawn;i++){			
				if (!enemyship[i].alive ) {					
					if (CountEnemyShip()+1 < rand()%MAX_ENEMYSHIP){ // Check if should spawn enemy or not
						EnemyReinit(enemyship[i]);	
					}
				}
			}
		}
}


void Enemy2Spawn() //! Handles the spawning of enemyship2
{
	int i,j;	
	for (i=0; i<MAX_ENEMYSHIP2; i++){								
		if (boundaryCheckDel(enemyship2[i]) == TRUE){
			enemyship2[i].alive = false;
			}		
	}	
	EnemySpawnProb();  	
	if (EnemySpawnProb() == true) {	
		int howmanytospawn = (rand()%MAX_ENEMYSHIP2)-1;
		for (i=0;i<howmanytospawn;i++){	
			if (!enemyship2[i].alive ) {
				if (CountEnemy2Ship()+1 < rand()%MAX_ENEMYSHIP2){
					EnemyReinit(enemyship2[i]);		
				}
			}
		}
	}
}	
						
	
int CountEnemyShip() // counts how many active asteroids we have
 {
	 int active=0;
	 int i;
	 for (i=0; i<MAX_ENEMYSHIP; i++){
		 if (enemyship[i].alive == true){
			 active++;
		 }
	 }
	 return active;
 }
int CountEnemy2Ship() // counts how many active asteroids we have
 {
	 int active=0;
	 int i;
	 for (i=0;i<MAX_ENEMYSHIP2;i++){
		 if (enemyship2[i].alive == true){
			 active++;
		 }
	 }
	 return active;
 }

bool EnemySpawnProb() // some function to see if we need to spawn more asteroid or not
{
	if( rand()%1000 < CHANCE_TO_SPAWN_ENEMY ){
		return TRUE;
	}
	else return FALSE;
}

void EnemyReinit(Entity &ent) // Reinitialises the asteroids after they are destroyed
{							// Change attributes like speed of roids here
	ent.alive = true;		
	int chance = (rand()%1000+rand()%1000)/2;
	if (chance <= 400){ // Spawns from top at any point

		ent.x = rand()%(800-ent.tileWidth);
		ent.y = ent.tileHeight;		
		int rand1 = rand()%10;
		if (rand1 <=5){
			ent.vx = (rand()%11)+1; 
			ent.vy = (rand()%12)+1;
		}
		if (rand1 >5){
			ent.vx = (rand()%8)+1;
			ent.vy = (rand()%9)+1;
		}
	}
	if (chance > 400 && chance <= 700){ // Spawns from left side at any point			
		ent.x = -ent.tileWidth;
		ent.y = rand()%WIN_HGT;		
		int rand1 = rand()%10;
		if (rand1 <=5){
			ent.vx = (rand()%9)+1; 
			ent.vy = -(rand()%7)+1;
		}
		if (rand1 >5){
			ent.vx = ((rand()%11)+1)*(-1);
			ent.vy = (rand()%12)+1;
		}
	}
	if (chance > 700 && chance <= 1000){ // Spawns from right side at any point
		ent.x = WIN_WID-ent.tileWidth;
		ent.y = rand()%WIN_HGT;		
		int rand1 = rand()%10;
		if (rand1 <=5){
			ent.vx = ((rand()%11)+1)*(-1); 
			ent.vy = (rand()%8)+1;
		}
		if (rand1 >5){
			ent.vx = ((rand()%9)+1)*(-1);
			ent.vy = (rand()%13)+1;
		}
	}
}

void SetEnemy1Bullet() // Places which enemy will shoot the bullet (A bit buggy at the moment, won't toggle for all enemies)
{
	int i;
	if (bullet_countdown_enemy > 0){
      bullet_countdown_enemy--;
      return;
	}
	
	int rand1 = rand()%MAX_ENEMYSHIP/2;

	for (i=0; i<MAX_ENEMYSHIP; i++){
		if(enemyship[i].alive == true){
			if(CountEnemyShip() < rand1){
				bullet_countdown_enemy = TIME_BETWEEN_ENEMY_BULLETS;
				insertEnemyBullet(enemyship[i]);
			}
		}
	}
}

void SetEnemy2Bullet() // Places which enemy will shoot the bullet (A bit buggy at the moment, won't toggle for all enemies)
{
	int i;
	if (bullet_countdown_enemy > 0){
      bullet_countdown_enemy--;
      return;
	}
	
	int rand1 = rand()%MAX_ENEMYSHIP2/2;

	for (i=0; i<MAX_ENEMYSHIP2; i++){
		if(enemyship2[i].alive == true){
			if(CountEnemyShip() < rand1){
				bullet_countdown_enemy = TIME_BETWEEN_ENEMY_BULLETS;
				insertEnemyBullet(enemyship2[i]);
			}
		}
	}
}


void MoveEnemyBullets()
{
	int y, i;	
	for (i=0; i < MAX_ENEMYBULLETS; i++){		
			if (eBullets[i].alive == true){
				y = eBullets[i].y; //! Save bullet y				
				y -= (eBullets[i].vy);   //! Move bullet         
				boundaryCheckDel(eBullets[i]);
				eBullets[i].y = y;				                  
				eBullets[i].duration--;
				if (eBullets[i].duration == 0 || (boundaryCheckDel(eBullets[i]) == true)){
					eBullets[i].alive = false;
				}
			}
	}
}
	
void insertEnemyBullet(Entity &ent) // Sets the enemy bullet properties
{
	int i;  	
	for (i=0; i < MAX_ENEMYBULLETS; i++){
		if(!eBullets[i].alive){		//! Insert new bullet if current one is dead			
			eBullets[i].x= ent.x + ent.tileWidth/2;
			eBullets[i].y= ent.y;
			eBullets[i].alive = true;  
			eBullets[i].vy=-(rand()%12 + rand()%5);
			eBullets[i].vx=-(rand()%12 + rand()%5);
			eBullets[i].duration = 18;  
			return;       
		}
	}
}
void InsertBullet() //! Inserts the bullet when user presses space bar
{   
	int i;  	
	if (bullet_countdown > 0){
      bullet_countdown--;
      return;
	}
		
	for (i=0; i < MAX_BULLETS; i++){
		if (!pBullets[i].alive) { //! Insert new bullet if current one is dead		
         pBullets[i].alive = true;
         pBullets[i].x = playership.x+15; 
         pBullets[i].y = playership.y;          
         pBullets[i].vy = 15;   //! Bullet speed
         pBullets[i].duration = 20; //! How long the ship bullet "appears" , kill bullet after this time
         bullet_countdown = TIME_BETWEEN_BULLETS;         
         return;         
      } 
	}  
} 

void MoveBullets()
{
	int y, i;
	for (i= 0; i< MAX_BULLETS; i++){
		if (pBullets[i].alive){
			y = pBullets[i].y; //! Save bullet y
			y -= pBullets[i].vy;   //! Move bullet         
			if (y < 0){
				pBullets[i].alive = false;
			}
			pBullets[i].y = y;			                  
			pBullets[i].duration--;
			if (pBullets[i].duration == 0) pBullets[i].alive = 0;
		}
	}
}

void ExplodeAllAsteroids()
{
	int i;
	for (i=0;i<MAX_ASTEROIDS;i++){	
		asteroids[i].alive = false;
		gScore+=250; // Score for each asteroids exploded
		InsertExplosion(asteroids[i]);
		}	
}

void ExplodeAllEnemies()
{
	int i,j;	
	for (i=0; i<MAX_ENEMYSHIP;i++){
		for (j=0; j<MAX_ENEMYSHIP2;j++){
			if(enemyship[i].alive == true || enemyship2[j].alive == true){
				gScore+=1200; // How much score to add for each alive ship
				enemyship[i].alive = false;
				enemyship2[j].alive = false;
				InsertExplosion(enemyship[i]);
				InsertExplosion(enemyship2[i]);
			}
		}
	}
}

void ResetAll()
{

   int i;
         
   for (i = 0; i < MAX_ASTEROIDS; i++) asteroids[i].alive = false;

   for (i = 0; i < MAX_BULLETS; i++) pBullets[i].alive = false;
     
   for (i = 0; i < MAX_ENEMYBULLETS; i++) eBullets[i].alive = false;

   for (i = 0; i < MAX_NUMBER_EXPLOSIONS; i++) explosion[i].alive = false;

   for (i = 0; i < MAX_ENEMYSHIP; i++) enemyship[i].alive = false;

   for (i = 0; i < MAX_ENEMYSHIP2; i++) enemyship2[i].alive = false;
}

//---------------------------------------End of Game AI------------------------------------------------


// --------------------------------------------------------------------
// Collision Detect
// --------------------------------------------------------------------



void PlayerBulAsteroidCol() // Collision Detection and Response for player bullet and asteroids
{
	int i, j;
	for (i=0; i<MAX_BULLETS; i++){
		for (j=0; j<MAX_ASTEROIDS; j++){
			if (CollisionDetect(pBullets[i], asteroids[j]) == true){
				gScore+=20;					
				InsertExplosion(asteroids[j]);
				pBullets[i].alive = false;	
			}
		}
	}return;
}
		

void PlayerBulEnemy1Col() // Collision detection and response for player bullet and Enemy
{
	int i, j;	
	for (i=0; i<MAX_BULLETS; i++){
		for (j=0; j<MAX_ENEMYSHIP; j++){				
				if (CollisionDetect(pBullets[i], enemyship[j]) == true ){					
					gScore+=200;	
					pBullets[i].alive = false;
					InsertExplosion(enemyship[j]);					
					InsertBonusProb();
					if (InsertBonusProb() == true){							
						InsertBonusSpawn(enemyship[j]);
					}
				}
		}
	}return;
}		

void PlayerBulEnemy2Col() // Collision detection and response for player bullet and Enemy
{
	int i, j;
	for (i=0; i<MAX_BULLETS; i++){
		for (j=0; j<MAX_ENEMYSHIP2; j++){			
				if (CollisionDetect(pBullets[i], enemyship2[j]) == true ){
					gScore+=150;
					pBullets[i].alive = false;
					InsertExplosion(enemyship2[j]);
						if (InsertBonusProb() == true){	
						InsertBonusSpawn(enemyship2[j]);
						}
				}
		}
	}
}				
	
	
void Enemy1BulletPlayerCol()
{
	int i;
	
	for (i=0; i<MAX_ENEMYBULLETS; i++){
		if (CollisionDetect(eBullets[i], playership) == true ){				
				gScore-=25;
				gHealth-=15;
				eBullets[i].alive = false;
				InsertExplosion(eBullets[i]);
		}		
	}
}
	

void Enemy2BulletPlayerCol()
{
	int i;
	for (i=0; i<MAX_ENEMYBULLETS; i++){
		if (CollisionDetect(eBullets[i], playership) == true ){
			gScore-=20;
			gHealth-=15;
			eBullets[i].alive = false;	
			InsertExplosion(eBullets[i]);
		}
	}
	return;
}

void PlayerBonus1() // Gives 5000 bonus score
{
	int i;

	for(i=0; i<MAX_BONUS_SCORE; i++){
		if(bonusScore[i].alive == true){
			if (CollisionDetect (playership, bonusScore[i])){
				bonusScore[i].alive = false;
				gScore+=5000;
				gHealth+=20;
				return;
			}
		}
	}
}

void PlayerBonus2() // Blows all rocks up if player gets this bonus adds mad score
{
	int i,j,k;
	
		for(j=0; j<MAX_HAZARD; j++){
			if(hazard[j].alive == true){
				if (CollisionDetect(playership, hazard[j])){
					hazard[j].alive = false;
					ExplodeAllAsteroids();
					ExplodeAllEnemies();
				}
			}
		}
		return;
}





void RoidPlayerCol() // Collision detection and response for player and asteroid collision
{
	int i;
	for (i=0; i<MAX_ASTEROIDS; i++){		
			if (CollisionDetect(asteroids[i], playership) == true ){
				gScore-=20;
				gHealth -=20; //how much health to reduce if player hits asteroids
				asteroids[i].alive= false;											
				InsertExplosion(asteroids[i]);	
			}		
	}
}
	
void EnemyPlayerCol() // Collision Detection and Response for enemy type 1 and player 
{                     // Enemy takes 2 hits to die 
	int i;
	for (i=0; i<MAX_ENEMYSHIP; i++){				
		if (CollisionDetect(playership,enemyship[i]) == true){					
			gScore-=50;
			gHealth-=5;	
			InsertExplosion(enemyship[i]);
			InsertBonusProb();	
			if (InsertBonusProb() == true){
				InsertBonusSpawn(enemyship[i]);
			}
		}
	}
}					

void Enemy2PlayerCol() // Collision Detection and Response for enemy type 2 and player 
{                     // Enemy takes 4 hits to die 
	int i;
	for (i=0; i<MAX_ENEMYSHIP; i++){	
		if (CollisionDetect(playership,enemyship2[i]) == true){
			gScore-=50;
			gHealth-=50;
			InsertExplosion(enemyship2[i]);	
			InsertBonusProb();	
			if (InsertBonusProb() == true){
				InsertBonusSpawn(enemyship2[i]);
			}
		}		
	}
}

		

void boundaryCheck(Entity &ent){ //! Used to make sure the object doesn't go out of the window
	
	if (ent.y < 0) {
		ent.y = 0;
	}
	if (ent.y > REAL_BOTTOM - ent.tileHeight) {
		ent.y = REAL_BOTTOM - ent.tileWidth;
	}
	if (ent.x < 0) {
		ent.x = 0;
	}
	if (ent.x > WIN_WID - (ent.tileHeight+7)) { // ent.w + 7 is some adjustments to make the boundary check better
		ent.x = WIN_WID - (ent.tileHeight+7);
	}	
}


 
bool CollisionDetect(Entity &ent1, Entity &ent2) // Returns true if there is a collision
{
	// Hit can only happen between two live objects
	if(!ent1.alive || !ent2.alive)
		return false;

	int halfw1 = ent1.tileWidth/2;
	int halfh1 = ent1.tileHeight/2;
	int halfw2 = ent2.tileWidth/2;
	int halfh2 = ent2.tileHeight/2;
	if ((ent1.x + halfw1) < (ent2.x - halfw2)) return false;
	if ((ent1.x - halfw1) > (ent2.x + halfw2)) return false;
	if ((ent1.y + halfh1) < (ent2.y - halfh2)) return false;
	if ((ent1.y - halfh1) > (ent2.y + halfh2)) return false;
	return true;
}

//----------------------------------------------End of Collision Detect------------------------------------------

// --------------------------------------------------------------------
// Rendering
// --------------------------------------------------------------------


void printStuff(int x, int y, const char *str, COLORREF color, 
	int s, const char *texttype, int fontweight, DWORD ital, DWORD underline, 
	int para)
{
	long size = -MulDiv(s, GetDeviceCaps(gBuffer.hdcBack, LOGPIXELSY), 72);
	HFONT hFont = CreateFont(size, 0, 0, fontweight, ital, underline, 
		0, 0, 0, 0, 0, 0, 0, texttype);
	HFONT hOldFont= (HFONT)SelectObject(gBuffer.hdcBack, hFont);
 
	SetBkColor(gBuffer.hdcBack, RGB(0,0,0));
	SetTextColor(gBuffer.hdcBack, color);
	SetBkMode(gBuffer.hdcBack, TRANSPARENT);
 
	char buffer[120];
	wsprintf(buffer, str, para);
	TextOut(gBuffer.hdcBack, x, y, buffer, strlen(buffer));
 
	SelectObject(gBuffer.hdcBack, hOldFont);
	DeleteObject(hFont);
}

void ShowScore() // Shows final score and some nice comments, called in GAME_OVER state
{ 
	ScrollBackground(gBuffer);
	printStuff(240, 50, "GAME OVER ", RGB(255,0 , 255), 38,  "Arial", 200, TRUE,0,  gScore);
	printStuff(220, 200, "Your Final Score is: %d ", RGB(255,0 , 255), 28, "Arial", 200, TRUE, 0 , gScore);
	printStuff(268, 300, "Play Again? (Y or N)", RGB(255, 0, 255), 22, "Arial", 200, TRUE,0, gScore);	
}

void entsAnimate(Entity ents[], int count)
{
	int i;
	for (i=0; i<count; i++) {		
		entityAnimate(ents[i]);
		entityMove(ents[i]);
		entityRender(ents[i]); 		
	}
}

void entsAnimateOnce(Entity ents[], int count)
{
	int i;
	for (i=0; i<count; i++) {		
		entityAnimateOnce(ents[i]);
		entityRender(ents[i]); 
	}
}

void GameRender()
{
	entityRender(life[0]);
	entityRender(life[1]);
	entityRender(life[2]);

	entityRender(fullhealth);
	entityRender(health80);
	entityRender(health60);
	entityRender(health40);
	entityRender(health20);
	entityRender(healthzero);
	
	entsAnimateOnce(explosion, MAX_NUMBER_EXPLOSIONS);

	entsAnimate(&playership, 1);
	entsAnimateOnce(pBullets, MAX_BULLETS);

	entsAnimate(enemyship, MAX_ENEMYSHIP);
	entsAnimate(enemyship2, MAX_ENEMYSHIP2);
	entsAnimate(asteroids, MAX_ASTEROIDS);

	entsAnimate(bonusScore, MAX_BONUS_SCORE);
	entsAnimate(hazard, MAX_HAZARD);

	entsAnimateOnce(eBullets, MAX_ENEMYBULLETS);

	
}


void healthbar() // Our healthbar function
{
	fullhealth.alive = true;
	if (gHealth == 300 && gScore > 280){        
		fullhealth.alive = true;
		health80.alive = false;
		health60.alive = false;
		health40.alive = false;
		health20.alive = false;
		healthzero.alive = false;
	}

	if (gHealth<=280 && gHealth>260){
		fullhealth.alive = false;
		health20.alive = false;
		health40.alive = false;
		health60.alive = false;
		health80.alive = true;
		healthzero.alive = false;
	}
	if (gHealth<=260 && gHealth>240){
		fullhealth.alive = false;
		health20.alive = false;
		health40.alive = false;
		health60.alive = true;
		health80.alive = false;
		healthzero.alive = false;
	}
	if (gHealth<=240 && gHealth>220){
		fullhealth.alive = false;
		health20.alive = false;
		health40.alive = true;
		health60.alive = false;
		health80.alive = false;
		healthzero.alive = false;
	}
	if (gHealth<=220 && gHealth>200){
		fullhealth.alive = false;
		health20.alive = true;
		health40.alive = false;
		health60.alive = false;
		health80.alive = false;
		healthzero.alive = false;
	}
	if (gHealth==0){
		fullhealth.alive = false;
		health20.alive = false;
		health40.alive = false;
		health60.alive = false;
		health80.alive = false;
		healthzero.alive = true;
	}
	if (gHealth <= 200 && gHealth >180){
		fullhealth.alive = true;
		health20.alive = false;
		health40.alive = false;
		health60.alive = false;
		health80.alive = false;
		healthzero.alive = false;
	}

	if (gHealth<=180 && gHealth>160){
		fullhealth.alive = false;
		health80.alive = true;
		health60.alive = false;
		health40.alive = false;
		health20.alive = false;
		healthzero.alive = false;
	}
	if (gHealth<=160 && gHealth>140){
		health80.alive = false;
		health60.alive = true;
	}
	if (gHealth<=140 && gHealth>120){
		health60.alive = false;
		health40.alive = true;
	}
	if (gHealth<=120 && gHealth>100){
		health40.alive = false;
		health20.alive = true;
	}
	if (gHealth <=100 && gHealth>80){
		fullhealth.alive = true;
		health20.alive = false;
		health40.alive = false;
		health60.alive = false;
		health80.alive = false;
		healthzero.alive = false;
	}
	if (gHealth<=80 && gHealth>60){
		fullhealth.alive = false;
		health20.alive = false;
		health40.alive = false;
		health60.alive = false;
		health80.alive = true;
		healthzero.alive = false;
	}
	if (gHealth<=60 && gHealth>40){
		fullhealth.alive = false;
		health20.alive = false;
		health40.alive = false;
		health60.alive = true;
		health80.alive = false;
		healthzero.alive = false;
	}
	if (gHealth<=40 && gHealth>20){
		fullhealth.alive = false;
		health20.alive = false;
		health40.alive = true;
		health60.alive = false;
		health80.alive = false;
		healthzero.alive = false;
	}
	if (gHealth<=20 && gHealth>0){
		fullhealth.alive = false;
		health20.alive = true;
		health40.alive = false;
		health60.alive = false;
		health80.alive = false;
		healthzero.alive = false;
	}
}


//-----------------------------------------------End of Rendering-----------------------------------



