#ifndef _GAME_H				
#define _GAME_H	

#include <windows.h>
#include "bitmap.h"

// --------- GAME SETTINGS ------------

#define BKGRD_WID 300 // Width of the background image in pixels
#define BKGRD_HGT 150 // Height of the background image in pixels

// Width and height of the game window
#define WIN_WID 800
#define WIN_HGT 600

#define REAL_BOTTOM				500 // the exact bottom limit of the window (the top menu bar takes up some space)
#define FRAME_RATE				30	// Set the frame rate to 30 frames per second
#define TIME_BETWEEN_BULLETS	2	// Bullet intervals
#define TIME_BETWEEN_ENEMY_BULLETS	6 // Enemy bullet intervals
#define MAX_BULLETS				8	// Maximum number of bullets at a time
#define MAX_ENEMYBULLETS		100	// Maximum number of bullets the enemy can fire
#define MAX_ENEMYSHIP			6	// Maximum number of enemyships at a time (x of each type)
#define MAX_ENEMYSHIP2			6	// Maximum number of enemyships at a time (x of each type)
#define MAX_ASTEROIDS			5	// Maximum number of asteroids at a time (must be more than 2!)
#define CHANCE_TO_SPAWN_ENEMY	460 // Out of 1000
#define MAX_NUMBER_EXPLOSIONS	50	// MUST be more than the total of asteroids, ship1 and ship2
#define CHANCE_SPAWN_BONUS		900 // Chance to spawn bonus out of 1000
#define MAX_BONUS_SCORE			6	// Maximum number of health bonuses that could appear
#define HEALTH_BONUS_DURATION	100	// How long the health bonus will stay alive if it isn't blown up 
#define MAX_HAZARD				6	// Maximum number of "hazard" bonus, this will blow up all asteroids on the screen
#define HAZARD_BONUS_DURATION	150 // How long the hazard bonus will stay alive if it isn't blown up

#define GAME_INTRO	0
#define GAME_INIT	1
#define GAME_RUN	2
#define GAME_PAUSE	3
#define GAME_STATE_GAME_PAUSE 4
#define GAME_OVER	5
#define GAME_EXIT	6

extern bool keys[256];		//!< keystates
extern HFONT font; //!< Font struct
extern BUFFER gBuffer;		//!< Create the double buffering structure

typedef struct Entity{
	int x, y;
	int vx, vy;
	int duration;
	int w, h;
	int health;
    bool alive;
	float aniIndex;
	float aniRate;
	int tileWidth;
	int tileHeight;
	HBITMAP bitmap;
}Entity;

/*! Initialises the objects for Struct Entity. Properties from left to right
	Name of object, x-coord,y-coord, vel-x,vel-y,1/alive 0/dead, health
	Width of bitmap rendered, Height of bitmap rendered, animation rate,
	reference to the bitmap array which the object is saved
*/
void entityInit(Entity &ent, int x, int y, int vx, int vy, bool alive, int health, 
	int tileWidth, int tileHeight,float aniRate, HBITMAP bitmap );


//! General functions for rendering, animation and movement
void entityMove(Entity &ent);
void entityRender(const Entity &ent);
void entityAnimate(Entity &ent);
void entityAnimateOnce(Entity &ent);
void entityRender(const Entity &ent);


// --------------------------------------------------------------------
// Globals
// --------------------------------------------------------------------


extern int bullet_countdown;
extern int bullet_countdown_enemy;

extern int Spawntimer;			// Delay for spawning stuff after player resets
extern int timer2;				// Do not change for the moment, used for final playership explosion delay time 
extern int gYScroll;
extern int gScore;				//Player score
extern int gHealth;
extern int countdown;
extern bool explodedOnce;
extern bool explodedTwice;
extern bool explodedThrice;

extern int BigExplodeDelay;	// Player ship explosion delay before starting new loop
extern char texttype[100];		// for our printStuff function how many strings to store
extern char textout[500];



// --------------------------------------------------------------------
// Prototypes
// --------------------------------------------------------------------

/*	Initialises the objects for Struct Entity. Properties from left to right
	 Name of object, x-coord,y-coord, vel-x,vel-y,1/alive 0/dead, health
	Width of bitmap rendered, Height of bitmap rendered, animation rate,
	General functions for rendering, animation and movement    				
	reference to the bitmap array which the object is saved
*/
void entityInit(Entity &ent, int x, int y, int vx, int vy, bool alive, int health, 
				int tileWidth, int tileHeight,float aniRate, HBITMAP bitmap );  																			    
																				
//	General functions for rendering, animation and movement    				
void entityMove(Entity &ent);
void entityRender(const Entity &ent);
void entityAnimate(Entity &ent);
void entityAnimateOnce(Entity &ent);
void entityRender(const Entity &ent);
SIZE GetBitmapSize(HBITMAP hbm);

void intro(); // Introduction function used in GAME_STATE_INTRO
bool GameInit();
void GameFrame();
void GameDeInit();

void PlayerSpawn();
void PlayerReset();
void InsertBullet();
void MoveBullets();
void InitLife(); 
void LifeCounter();
void PlayerExplosionCount();

void PauseAll();

void RoidReinit(Entity &ent);
int CountAsteroids();
void AsteroidSpawn();
bool RoidSpawnProb();
void ExplodeAllAsteroids();

bool EnemySpawnProb();
void InsertEnemyship(Entity ent[], int x, int y);
void InsertEnemy1Bullet();
void InsertEnemy2Bullet();
void MoveEnemyBullets();
void MoveEnemyship();
void EnemySpawn();
void Enemy2Spawn();

void InsertExplosion(Entity &ent);
void SpawnExplosion();

void InsertBonusSpawn(Entity &ent);
void SpawnBonus();
void PlayerBonus1();
void PlayerBonus2();
bool InsertBonusProb();

void EnemyReinit(Entity &ent);
int CountEnemyShip();
int CountEnemy2Ship();
void ExplodeAllEnemies();
void SetEnemy1Bullet();
void SetEnemy2Bullet();
void insertEnemyBullet(Entity &ent);

bool CollisionDetect(Entity &ent1, Entity &ent2);
void RoidPlayerCol();
void PlayerBulAsteroidCol();
void PlayerBulEnemy1Col();
void Enemy1BulletPlayerCol();
void PlayerBulEnemy2Col();
void Enemy2BulletPlayerCol();
void EnemyPlayerCol(); 
void Enemy2PlayerCol();
bool boundaryCheckDel(Entity &ent);
void boundaryCheck(Entity &ent);

void ScrollBackground(BUFFER &buffer);
void printScore();
void printHealth();
void entsAnimate(Entity ents[], int count);
void entsAnimateOnce(Entity ents[], int count);
void healthbar();
void ShowScore();
void GameRender();


/*!	Print text function, list of parameters from left to right:
	x-pos,y-pos, text to print, RGB color, font size, name of font, 
	400-normal,700=bold, Italics(0 false,TRUE true), 
	Underline(0 false, TRUE true), parameters for the textoutput, if
	needed, for ex. "FOO %d", type 0 in here is not needed 
	Unfortunately for now the parameters MUST be integers 
*/
void printStuff(int x, int y, const char *textout,																		
				COLORREF color,int s, const char *texttype,				
				int fontweight ,DWORD ital, DWORD underline, int para);     
long GetRectHeight(const RECT &rect);
long GetRectWidth(const RECT &rect);
void SystemInit(HWND hwnd);
void SystemDeInit();
bool LockFrameRate(int desired_frame_rate);
void ResetAll();


#endif